"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.EvaluationService = void 0;
var moment = require("moment");
var log4js = require("log4js");
var mssql_service_1 = require("../db/mssql.service");
var promises_shared_1 = require("../shared/promises.shared");
var redis_service_1 = require("../db/redis.service");
var evaluation_result_model_1 = require("../models/evaluation-result.model");
var evaluation_model_1 = require("../models/evaluation.model");
var utilities_1 = require("../shared/utilities");
var role_service_1 = require("./role.service");
var logger = log4js.getLogger('ServerScripts');
function EvaluationService(app) {
    app.post('/api/evaluation/get_evaluation_results/', function (req, res, next) {
        logger.info('[EvaluationService::get_evaluation_results] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.instance;
        var userId = res.locals.userId;
        var evaluationPlanId = req.body.evaluationPlanId || null;
        var interactionId = req.body.interactionId || null;
        var auditorId = req.body.auditorId || userId || null;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return mssql_service_1.MSSql.GetEvaluationResults(vcc, evaluationPlanId, interactionId, auditorId);
        }).then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        }).catch(function (err) {
            var msg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[EvaluationService::get_evaluation_results] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: msg, error: errorMsg });
        });
    });
    app.post('/api/evaluation/save_evaluation_status', function (req, res, next) {
        logger.info('[EvaluationService::save_evaluation_status] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.instance;
        var userId = res.locals.userId;
        var evaluationToUpdate = (req.body.evaluation || {});
        evaluationToUpdate.auditorId = userId;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return redis_service_1.Redis.SaveEvaluationStatus(vcc, evaluationToUpdate);
        })
            .then(function (result) {
            if (result && result.status === false)
                throw result.error;
            if (res.headersSent)
                return;
            logger.info('[EvaluationService::save_evaluation_status] The evaluation update has been finished.');
            res.json({ status: true, description: (result ? 'SERVER_SUCCESS_MESSAGE' : 'SERVER_ERROR_UNKNOWN_MESSAGE'), data: true });
        })
            .catch(function (err) {
            var msg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Executing request error');
            logger.error('[EvaluationService::save_evaluation_status] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: msg, error: errorMsg });
        });
    });
    app.post('/api/evaluation/discard_evaluation_status', function (req, res, next) {
        logger.info('[EvaluationService::save_evaluation_status] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.instance;
        var userId = res.locals.userId;
        var evaluationToUpdate = (req.body.evaluation || {});
        evaluationToUpdate.auditorId = userId;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return redis_service_1.Redis.DeletEvaluationStatus(vcc, evaluationToUpdate);
        })
            .then(function (result) {
            if (result && result.status === false)
                throw result.error;
            if (res.headersSent)
                return;
            logger.info('[EvaluationService::discard_evaluation_status] The evaluation update has been finished.');
            res.json({ status: true, description: (result ? 'SERVER_SUCCESS_MESSAGE' : 'SERVER_ERROR_UNKNOWN_MESSAGE'), data: true });
        })
            .catch(function (err) {
            var msg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Executing request error');
            logger.error('[EvaluationService::discard_evaluation_status] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: msg, error: errorMsg });
        });
    });
    app.post('/api/evaluation/save_evaluationresult', function (req, res) {
        logger.info('[EvaluationService::save_evaluationresult] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.instance;
        var userId = res.locals.userId;
        var evaluations = (req.body || {});
        if (evaluations.isDraft) {
            evaluations.evaluationResult = evaluations.evaluationResult.filter(function (er) { return er.hasOwnProperty('qualification') && er.qualification !== null; });
        }
        var saveResult = null;
        promises_shared_1.DummyPromise()
            .then(function (_) {
            if (evaluations.evaluationResult.length === 0)
                throw 'SERVER_ERROR_EVALUATION_RESULTS_NOT_RECEIVED';
            return mssql_service_1.MSSql.GetAuditorEvaluationPlanEvaluation(vcc, userId, evaluations.evaluationResult[0].idEvaluationPlan, evaluations.evaluationResult[0].idInteraction, evaluations.evaluationResult[0].interactionType);
        })
            .then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_EVALUATION_NOT_FOUND';
            if (result.completed || !result.status) {
                throw 'SERVER_ERROR_EVALUATION_IS_NOT_AVAILABLE';
            }
            else {
                return SaveEvaluationResults(vcc, userId, result.evaluation_mode, evaluations);
            }
        })
            .then(function (result) {
            saveResult = result;
            var evaluationToRemove = new evaluation_model_1.EvaluationToUpdate();
            evaluationToRemove.evaluationPlanId = evaluations.evaluationResult[0].idEvaluationPlan;
            evaluationToRemove.interactionId = evaluations.evaluationResult[0].idInteraction;
            evaluationToRemove.auditorId = userId;
            return redis_service_1.Redis.DeletEvaluationStatus(vcc, evaluationToRemove);
        })
            .then(function (result) {
            res.json({ status: true, description: (saveResult ? 'SERVER_SUCCESS_MESSAGE' : 'Evaluation Results save failed'), data: saveResult });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[EvaluationService::save_evaluationresult] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/evaluation/discard_evaluation_draft', function (req, res, next) {
        logger.info('[EvaluationService::discard_evaluation_draft] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.instance;
        var userId = res.locals.userId;
        var evaluationPlanId = req.body.evaluation.evaluationPlanId || null;
        var interactionId = req.body.evaluation.interactionId || null;
        promises_shared_1.DummyPromise()
            .then(function (_) {
            return mssql_service_1.MSSql.RestoreEvaluationResultDraftBackup(vcc, evaluationPlanId, interactionId, userId);
        })
            .then(function (result) {
            if (result && result.status === false)
                throw result.error;
            if (res.headersSent)
                return;
            logger.info('[EvaluationService::discard_evaluation_draft] The evaluation draft have been deleted.');
            res.json({ status: true, description: (result ? 'SERVER_SUCCESS_MESSAGE' : 'SERVER_ERROR_UNKNOWN_MESSAGE'), data: true });
        })
            .catch(function (err) {
            var msg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Executing request error');
            logger.error('[EvaluationService::discard_evaluation_draft] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: msg, error: errorMsg });
        });
    });
    app.get('/api/evaluation/get_interaction_evaluation/:interaction?', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var instance = res.locals.instance;
        var interaction = req.params.interaction || '';
        var interactionType = req.query.interactionType || '';
        var evaluationDateFrom = req.query.evaluationDateFrom || '1980-01-01';
        var evaluationDateTo = req.query.evaluationDateTo || '2050-01-01';
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return mssql_service_1.MSSql.GetEvaluationsOfInteraction(instance, interaction, interactionType, evaluationDateFrom, evaluationDateTo);
        }).then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        }).catch(function (err) {
            var msg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: msg, error: errorMsg });
        });
    });
    app.post('/api/evaluation/get_evaluation_agent/', function (req, res, next) {
        logger.info('[EvaluationService::get_evaluation_agent] Received ' + req.method + ' method on \'' + req.path + '\'');
        var instance = res.locals.instance;
        var idAgent = res.locals.userId;
        logger.info('[EvaluationService::get_evaluation_agent] Filling the filter\'s object');
        var params = new evaluation_result_model_1.EvaluationResultFilters;
        params.dateIni = req.body.dateIni || '';
        params.dateFin = req.body.dateFin || '';
        params.campaigns = req.body.campaigns || [];
        params.evaluationPlans = req.body.evaluationPlans || [];
        params.origins = req.body.origins || [];
        params.minDuration = req.body.minDuration || null;
        params.maxDuration = req.body.maxDuration || null;
        params.evaluationTypes = req.body.evaluationTypes || [];
        params.tags = req.body.tags || [];
        params.hasVideoOptions = req.body.hasVideoOptions || [];
        params.results = req.body.results ? req.body.results.map(function (r) {
            var dispositionCode = null;
            if (typeof r === 'string') {
                dispositionCode = r;
            }
            else {
                dispositionCode = r.campaignId;
                if (r.description)
                    dispositionCode += '_' + r.description;
                if (r.description2)
                    dispositionCode += '_' + r.description2;
                if (r.description3)
                    dispositionCode += '_' + r.description3;
                if (r.description4)
                    dispositionCode += '_' + r.description4;
            }
            return dispositionCode;
        }) : [];
        params.hourIni = req.body.hourIni || '';
        params.hourFin = req.body.hourFin || '';
        params.multimediaFormats = ['mp3', 'wav'];
        if (params.hasVideoOptions.includes(true)) {
            params.multimediaFormats.push('flv');
        }
        params.origins = params.origins.map(function (o) {
            if (o == "E")
                return "IN";
            else if (o == "S")
                return "OUT";
            else
                return null;
        });
        params.interactionIds = req.body.interactionIds || [];
        params.contactAddress = req.body.contactAddress || '';
        logger.debug('[EvaluationService::get_evaluation_agent] Filter\'s object: ', JSON.stringify(params));
        promises_shared_1.DummyPromise()
            .then(function (result) {
            logger.info('[EvaluationService::get_evaluation_agent] Calling GetEvaluations SP for VCC|idAgent: ' + instance + '|' + idAgent);
            return mssql_service_1.MSSql.GetEvaluations(instance, null, idAgent, params.campaigns.toString(), params.evaluationPlans.toString(), params.dateIni, params.dateFin, params.hourIni, params.hourFin, params.minDuration, params.maxDuration, params.origins.toString(), params.results.toString(), params.evaluationTypes.toString(), params.tags.toString(), params.multimediaFormats.toString(), null, params.interactionIds.toString(), params.contactAddress.toString());
        })
            .then(function (result) {
            logger.debug('[EvaluationService::get_evaluation_agent] Received result from GetEvaluations SP: ' + JSON.stringify(result));
            if (res.headersSent)
                return;
            logger.info('[EvaluationService::get_evaluation_agent] Sending results: ' + result.length);
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch(function (err) {
            var msg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Requesting error');
            logger.error('[EvaluationService::get_evaluation_agent] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: msg, error: errorMsg });
        });
    });
    app.post('/api/evaluation/get_evaluation_auditor/', function (req, res, next) {
        logger.info('[EvaluationService::get_evaluation_auditor] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.instance;
        var idAuditor = res.locals.userId;
        var currentUserId = res.locals.userId;
        var currentUserRole = res.locals.user.role;
        logger.info('[EvaluationService::get_evaluation_auditor] Filling the filter\'s object');
        var params = new evaluation_result_model_1.EvaluationResultFilters;
        params.dateIni = req.body.dateIni || '';
        params.dateFin = req.body.dateFin || '';
        params.campaigns = req.body.campaigns || [];
        params.evaluationPlans = req.body.evaluationPlans || [];
        params.agents = req.body.agents || [];
        params.origins = req.body.origins || [];
        params.minDuration = req.body.minDuration || null;
        params.maxDuration = req.body.maxDuration || null;
        params.evaluationTypes = req.body.evaluationTypes && req.body.evaluationTypes.length ? [] : ['M', 'X'];
        params.tags = req.body.tags || [];
        params.hasVideoOptions = req.body.hasVideoOptions || [];
        params.coached = req.body.coached || [];
        params.results = req.body.results ? req.body.results.map(function (r) {
            var dispositionCode = null;
            if (typeof r === 'string') {
                dispositionCode = r;
            }
            else {
                dispositionCode = r.campaignId;
                if (r.description)
                    dispositionCode += '_' + r.description;
                if (r.description2)
                    dispositionCode += '_' + r.description2;
                if (r.description3)
                    dispositionCode += '_' + r.description3;
                if (r.description4)
                    dispositionCode += '_' + r.description4;
            }
            return dispositionCode;
        }) : [];
        params.hourIni = req.body.hourIni || '';
        params.hourFin = req.body.hourFin || '';
        params.multimediaFormats = ['mp3', 'wav'];
        if (params.hasVideoOptions.includes(true)) {
            params.multimediaFormats.push('flv');
        }
        params.origins = params.origins.map(function (o) {
            if (o == "E")
                return "IN";
            else if (o == "S")
                return "OUT";
            else
                return null;
        });
        params.interactionIds = req.body.interactionIds || [];
        params.contactAddress = req.body.contactAddress || '';
        logger.debug('[EvaluationService::get_evaluation_auditor] Filter\'s object: ', JSON.stringify(params));
        promises_shared_1.DummyPromise()
            .then(function (_) {
            return role_service_1.GetRoleFunctions(vcc, currentUserId, currentUserRole);
        })
            .then(function (result) {
            logger.info('[EvaluationService::get_evaluation_auditor] Calling GetEvaluations SP for VCC|IdAuditor: ' + vcc + '|' + currentUserId);
            if (result.data.hasSupervisorFunctions)
                idAuditor = null;
            return mssql_service_1.MSSql.GetEvaluations(vcc, idAuditor, params.agents.toString(), params.campaigns.toString(), params.evaluationPlans.toString(), params.dateIni, params.dateFin, params.hourIni, params.hourFin, params.minDuration, params.maxDuration, params.origins.toString(), params.results.toString(), params.evaluationTypes.toString(), params.tags.toString(), params.multimediaFormats.toString(), params.coached.toString(), params.interactionIds.toString(), params.contactAddress.toString());
        })
            .then(function (result) {
            logger.debug('[EvaluationService::get_evaluation_auditor] Received result from GetEvaluations SP: ' + JSON.stringify(result));
            if (res.headersSent)
                return;
            logger.info('[EvaluationService::get_evaluation_auditor] Sending results: ' + result.length);
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch(function (err) {
            var msg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error requesting');
            logger.error('[EvaluationService::get_evaluation_auditor] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: msg, error: errorMsg });
        });
    });
    app.post('/api/evaluation/get_evaluation_admin/', function (req, res, next) {
        logger.info('[EvaluationService::get_evaluation_admin] Received ' + req.method + ' method on \'' + req.path + '\'');
        var instance = res.locals.instance;
        logger.info('[EvaluationService::get_evaluation_admin] Filling the filter\'s object');
        var params = new evaluation_result_model_1.EvaluationResultFilters;
        params.dateIni = req.body.dateIni || '';
        params.dateFin = req.body.dateFin || '';
        params.campaigns = req.body.campaigns || [];
        params.evaluationPlans = req.body.evaluationPlans || [];
        params.auditors = req.body.auditors || [];
        params.agents = req.body.agents || [];
        params.origins = req.body.origins || [];
        params.minDuration = req.body.minDuration || null;
        params.maxDuration = req.body.maxDuration || null;
        params.evaluationTypes = req.body.evaluationTypes || [];
        params.tags = req.body.tags || [];
        params.hasVideoOptions = req.body.hasVideoOptions || [];
        params.coached = req.body.coached || [];
        params.results = req.body.results ? req.body.results.map(function (r) {
            var dispositionCode = null;
            if (typeof r === 'string') {
                dispositionCode = r;
            }
            else {
                dispositionCode = r.campaignId;
                if (r.description)
                    dispositionCode += '_' + r.description;
                if (r.description2)
                    dispositionCode += '_' + r.description2;
                if (r.description3)
                    dispositionCode += '_' + r.description3;
                if (r.description4)
                    dispositionCode += '_' + r.description4;
            }
            return dispositionCode;
        }) : [];
        params.hourIni = req.body.hourIni || '';
        params.hourFin = req.body.hourFin || '';
        params.multimediaFormats = ['mp3', 'wav'];
        if (params.hasVideoOptions.includes(true)) {
            params.multimediaFormats.push('flv');
        }
        params.origins = params.origins.map(function (o) {
            if (o == "E")
                return "IN";
            else if (o == "S")
                return "OUT";
            else
                return null;
        });
        params.interactionIds = req.body.interactionIds || [];
        params.contactAddress = req.body.contactAddress || '';
        logger.debug('[EvaluationService::get_evaluation_admin] Filter\'s object: ', JSON.stringify(params));
        promises_shared_1.DummyPromise()
            .then(function (result) {
            logger.info('[EvaluationService::get_evaluation_admin] Calling GetEvaluations SP for VCC|IdAuditor: ' + instance);
            return mssql_service_1.MSSql.GetEvaluations(instance, params.auditors.toString(), params.agents.toString(), params.campaigns.toString(), params.evaluationPlans.toString(), params.dateIni, params.dateFin, params.hourIni, params.hourFin, params.minDuration, params.maxDuration, params.origins.toString(), params.results.toString(), params.evaluationTypes.toString(), params.tags.toString(), params.multimediaFormats.toString(), params.coached.toString(), params.interactionIds.toString(), params.contactAddress.toString());
        })
            .then(function (result) {
            logger.debug('[EvaluationService::get_evaluation_admin] Received result from GetEvaluations SP: ' + JSON.stringify(result));
            if (res.headersSent)
                return;
            logger.info('[EvaluationService::get_evaluation_admin] Sending results: ' + result.length);
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch(function (err) {
            var msg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: msg, error: errorMsg });
        });
    });
    app.post('/api/evaluation/get_evaluation_results_from_auditor_evaluation_plan/:evaluationPlanId', function (req, res, next) {
        logger.info('[EvaluationService::get_evaluation_results_from_auditor_evaluation_plan] Received ' + req.method + ' method on \'' + req.path + '\'');
        var instance = res.locals.instance;
        var auditorId = res.locals.userId;
        logger.info('[EvaluationService::get_evaluation_results_from_auditor_evaluation_plan] Filling the filter\'s object');
        var params = new evaluation_result_model_1.EvaluationResultInteractionFilters;
        params.evaluationPlanId = req.params.evaluationPlanId || '';
        params.interactionId = req.body.interactionId || '';
        params.auditorId = req.body.auditorId || '';
        params.agentId = req.body.agentId || '';
        params.interactionType = req.body.interactionType || '';
        var results = {};
        logger.debug('[EvaluationService::get_evaluation_results_from_auditor_evaluation_plan] Filter\'s object: ', JSON.stringify(params));
        promises_shared_1.DummyPromise()
            .then(function (result) {
            logger.info('[EvaluationService::get_evaluation_results_from_auditor_evaluation_plan] Calling GetEvaluationResultsFromAuditorEvaluationPlan SP for VCC|IdEP: ' + instance + '|' + params.evaluationPlanId);
            return mssql_service_1.MSSql.GetEvaluationResultsFromAuditorEvaluationPlan(instance, params.evaluationPlanId, params.interactionId, params.interactionType, params.auditorId, params.agentId);
        })
            .then(function (result) {
            results.evaluationResultData = result;
            logger.info('[EvaluationService::get_evaluation_results_from_auditor_evaluation_plan] Calling GetEvaluationCustomFieldResultsFromAuditorEvaluationPlan SP for VCC|IdEP: ' + instance + '|' + params.evaluationPlanId);
            return mssql_service_1.MSSql.GetEvaluationCustomFieldResultsFromAuditorEvaluationPlan(instance, params.evaluationPlanId, params.interactionId, params.auditorId, params.agentId);
        })
            .then(function (result) {
            results.evaluationCustomFieldResultData = result;
            return mssql_service_1.MSSql.GetEvaluationFormatAttachedSignatures(instance, results.evaluationResultData[0].EvaluationFormatId, params.evaluationPlanId);
        })
            .then(function (result) {
            results.attachedSignatures = result;
            if (res.headersSent)
                return;
            logger.info('[EvaluationService::get_evaluation_results_from_auditor_evaluation_plan] Received result from GetEvaluationResultsFromAuditorEvaluationPlan SP');
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: results });
        })
            .catch(function (err) {
            var msg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error requesting');
            logger.error('[EvaluationService::get_evaluation_results_from_auditor_evaluation_plan] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: msg, error: errorMsg });
        });
    });
    app.get('/api/evaluation/get_evaluation_reports/:evformatid?', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var instance = res.locals.instance;
        var idEvaluationFormat = req.params.evformatid || '';
        var idsCampanias = req.query.campaigns || null, idAgente = req.query.agent || null, fechaDesde = req.query.dateFrom || null, fechaHasta = req.query.dateTo || null, idAuditor = req.query.auditor || null, planAuditoria = req.query.evaluationPlanId || null;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return mssql_service_1.MSSql.ReportAuditsByAgent(instance, idEvaluationFormat, idsCampanias, idAgente, fechaDesde, fechaHasta, idAuditor, planAuditoria);
        }).then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        }).catch(function (err) {
            var msg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: msg, error: errorMsg });
        });
    });
    app.post('/api/evaluation/get_evaluation_reviews', function (req, res, next) {
        logger.info('[EvaluationService::get_evaluation_reviews] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var currentUserId = res.locals.userId;
        var currentUserRole = res.locals.user.role;
        var evaluationPlanId = req.body.evaluationPlanId || null;
        var interactionId = req.body.interactionId || null;
        var supervisorId = req.body.supervisorId || null;
        var agentId = req.body.agentId || null;
        var userHasOnlyAgentFunctions = false;
        promises_shared_1.DummyPromise()
            .then(function (_) {
            return role_service_1.GetRoleFunctions(vcc, currentUserId, currentUserRole);
        })
            .then(function (result) {
            if (result
                &&
                    result.data
                &&
                    (!result.data.hasAdminFunctions
                        &&
                            !result.data.hasAuditorFunctions
                        &&
                            result.data.hasAgentFunctions))
                userHasOnlyAgentFunctions = true;
            if (userHasOnlyAgentFunctions) {
                logger.info('[EvaluationService::get_evaluation_reviews] Current user has only agent function.');
                agentId = currentUserId;
            }
            logger.info("[EvaluationService::get_evaluation_reviews] Getting review for VCC|EvaluationPlan|InteractionId|IdAuditor|Agent: " + vcc + "|" + evaluationPlanId + "|" + interactionId + "|" + supervisorId + "|" + agentId);
            return mssql_service_1.MSSql.GetEvaluationReview(vcc, evaluationPlanId, interactionId, supervisorId, agentId);
        })
            .then(function (result) {
            logger.debug('[EvaluationService::get_evaluation_reviews] Received result from GetEvaluationReview SP: ' + JSON.stringify(result));
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch(function (err) {
            var msg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[EvaluationService::get_evaluation_reviews] Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: msg, error: errorMsg });
        });
    });
    app.post('/api/evaluation/save_evaluation_review', function (req, res, next) {
        logger.info('[EvaluationService::save_evaluation_review] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var currentUserId = res.locals.userId;
        var evaluationReview = req.body.evaluationReview || null;
        promises_shared_1.DummyPromise()
            .then(function (_) {
            logger.info("[EvaluationService::save_evaluation_review] Saving review for VCC|EvaluationPlan|InteractionId|IdAuditor: " + vcc + "|" + evaluationReview.evaluationPlanId + "|" + evaluationReview.interactionId + "|" + evaluationReview.supervisorId);
            return mssql_service_1.MSSql.SaveEvaluationReview(vcc, evaluationReview, currentUserId);
        })
            .then(function (result) {
            if (res.headersSent)
                return;
            logger.info('[EvaluationService::save_evaluation_review] The evaluation update has been finished.');
            res.json({ status: true, description: (result ? 'SERVER_SUCCESS_MESSAGE' : 'SERVER_ERROR_UNKNOWN_MESSAGE'), data: true });
        })
            .catch(function (err) {
            var msg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Executing request error');
            logger.error('[EvaluationService::save_evaluation_review] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: msg, error: errorMsg });
        });
    });
    app.post('/api/evaluation/get_evaluation_review_commitments', function (req, res, next) {
        logger.info('[EvaluationService::get_evaluation_review_commitments] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var currentUserId = res.locals.userId;
        var currentUserRole = res.locals.user.role;
        var agentId = req.body.agentId || null;
        var userHasOnlyAgentFunctions = false;
        promises_shared_1.DummyPromise()
            .then(function (_) {
            return role_service_1.GetRoleFunctions(vcc, currentUserId, currentUserRole);
        })
            .then(function (result) {
            if (result
                &&
                    result.data
                &&
                    (!result.data.hasAdminFunctions
                        &&
                            !result.data.hasAuditorFunctions
                        &&
                            result.data.hasAgentFunctions))
                userHasOnlyAgentFunctions = true;
            if (userHasOnlyAgentFunctions) {
                logger.info('[EvaluationService::get_evaluation_review_commitments] Current user has only agent function.');
                agentId = currentUserId;
            }
            logger.info("[EvaluationService::get_evaluation_review_commitments] Getting review for VCC|Agent: " + vcc + "|" + agentId);
            return mssql_service_1.MSSql.GetEvaluationReviewCommitments(vcc, null, null, null, agentId);
        })
            .then(function (result) {
            logger.debug('[EvaluationService::get_evaluation_review_commitments] Received result from GetEvaluationReview SP: ' + JSON.stringify(result));
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch(function (err) {
            var msg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[EvaluationService::get_evaluation_review_commitments] Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: msg, error: errorMsg });
        });
    });
    app.post('/api/evaluation/save_evaluation_review_commitment', function (req, res, next) {
        logger.info('[EvaluationService::save_evaluation_review_commitment] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var currentUserId = res.locals.userId;
        var evaluationReviewCommitment = req.body.evaluationReviewCommitment || null;
        evaluationReviewCommitment.supervisorId = currentUserId;
        promises_shared_1.DummyPromise()
            .then(function (_) {
            return mssql_service_1.MSSql.SaveEvaluationReviewCommitment(vcc, evaluationReviewCommitment, currentUserId);
        })
            .then(function (result) {
            if (res.headersSent)
                return;
            logger.info('[EvaluationService::save_evaluation_review_commitment] The evaluation update has been finished.');
            res.json({ status: true, description: (result ? 'SERVER_SUCCESS_MESSAGE' : 'SERVER_ERROR_UNKNOWN_MESSAGE'), data: true });
        })
            .catch(function (err) {
            var msg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Executing request error');
            logger.error('[EvaluationService::save_evaluation_review_commitment] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: msg, error: errorMsg });
        });
    });
}
exports.EvaluationService = EvaluationService;
function SaveEvaluationResults(vcc, userId, evaluationPlanMode, interactionEvaluationPlanResult) {
    logger.info('[EvaluationService::SaveEvaluationResults] interactionEvaluationPlanResult: ', interactionEvaluationPlanResult);
    var nowUtc = moment().utc().toDate();
    var evaluationResultPromises = [];
    var evaluationCustomFieldResultPromises = [];
    var ok;
    var evaluationPlanId;
    var interactionId;
    var interactionType;
    return promises_shared_1.DummyPromise()
        .then(function (result) {
        if (interactionEvaluationPlanResult && interactionEvaluationPlanResult.evaluationResult && interactionEvaluationPlanResult.evaluationResult.length > 0) {
            ok = true;
            evaluationPlanId = interactionEvaluationPlanResult.evaluationResult[0].idEvaluationPlan;
            interactionId = interactionEvaluationPlanResult.evaluationResult[0].idInteraction;
            interactionType = interactionEvaluationPlanResult.evaluationResult[0].interactionType;
            if (evaluationPlanMode === 'X' && interactionEvaluationPlanResult.isDraft)
                return mssql_service_1.MSSql.SaveEvaluationResultDraftBackup(vcc, evaluationPlanId, interactionType, interactionId, userId);
            else
                return Promise.resolve(true);
        }
        else {
            throw 'SERVER_ERROR_EVALUATION_RESULTS_NOT_FOUND';
        }
    })
        .then(function (result) {
        interactionEvaluationPlanResult.evaluationResult.forEach(function (newEvaluationResult) {
            if (!newEvaluationResult.id) {
                newEvaluationResult.id = utilities_1.guid();
                newEvaluationResult.createdDate = nowUtc;
                newEvaluationResult.createdByUserId = userId;
            }
            else {
                newEvaluationResult.createdDate = null;
                newEvaluationResult.createdByUserId = null;
            }
            newEvaluationResult.lastModifiedDate = nowUtc;
            newEvaluationResult.lastModifiedByUserId = userId;
            newEvaluationResult.idAuditor = userId;
            newEvaluationResult.evaluationDate = nowUtc;
            logger.debug('[EvaluationService::SaveEvaluationResults] Guardando resultado evaluacion:' + JSON.stringify(newEvaluationResult));
            evaluationResultPromises.push(mssql_service_1.MSSql.SaveEvaluationResult(vcc, newEvaluationResult.id, newEvaluationResult.idCampaign, newEvaluationResult.idInteraction, newEvaluationResult.idAgent, newEvaluationResult.idEvaluationFormat, newEvaluationResult.minQualificationAcceptable, newEvaluationResult.idGroup, newEvaluationResult.idSubGroup, newEvaluationResult.idAbility, newEvaluationResult.qualification, newEvaluationResult.ownScore, newEvaluationResult.inheritedScore, newEvaluationResult.idAuditor, newEvaluationResult.evaluationDate, newEvaluationResult.maxAbilityScore, newEvaluationResult.abilityInnerScore, newEvaluationResult.idEvaluationPlan, newEvaluationResult.comments, newEvaluationResult.notApplies, newEvaluationResult.scoreDescription, newEvaluationResult.campaignDescription, newEvaluationResult.agentDescription, new Date(newEvaluationResult.interactionDateini), new Date(newEvaluationResult.interactionDatefin), newEvaluationResult.interactionAddress, newEvaluationResult.interactionType, newEvaluationResult.abilityOrder, newEvaluationResult.createdDate, newEvaluationResult.createdByUserId, newEvaluationResult.lastModifiedDate, newEvaluationResult.lastModifiedByUserId, newEvaluationResult.conditionalSelected));
        });
        return Promise.all(evaluationResultPromises);
    })
        .then(function (result) {
        if (!result || (result && result.includes(false)))
            throw 'SERVER_ERROR_EVALUATION_RESULTS_SAVE_FAILED';
        if (interactionEvaluationPlanResult && interactionEvaluationPlanResult.evaluationCustomFieldResults.length > 0) {
            interactionEvaluationPlanResult.evaluationCustomFieldResults.map(function (ecfr) {
                ecfr.results.map(function (r) {
                    evaluationCustomFieldResultPromises.push(mssql_service_1.MSSql.SaveEvaluationCustomFieldResult(vcc, ecfr.evaluationPlanId, ecfr.interactionId, userId, ecfr.agentId, ecfr.customFieldId, r, userId, userId));
                });
            });
            return Promise.all(evaluationCustomFieldResultPromises);
        }
        else {
            return Promise.resolve([true]);
        }
    })
        .then(function (data) {
        return mssql_service_1.MSSql.SaveInteractionEvaluationPlanResult(vcc, evaluationPlanId, userId, interactionId, interactionType, interactionEvaluationPlanResult.interactionScore, interactionEvaluationPlanResult.interactionAutomaticScore, interactionEvaluationPlanResult.interactionManualScore, interactionEvaluationPlanResult.isDraft);
    })
        .then(function (data) {
        return mssql_service_1.MSSql.DeleteAuditorEvaluationPlanByInteraction(vcc, evaluationPlanId, interactionId, interactionType, interactionEvaluationPlanResult.isDraft);
    })
        .then(function (data) {
        return mssql_service_1.MSSql.SaveEvaluationPlanCompleted(vcc, evaluationPlanId);
    })
        .then(function (data) {
        if (evaluationPlanMode === 'X' && !interactionEvaluationPlanResult.isDraft)
            return mssql_service_1.MSSql.DeleteEvaluationResultDraftBackup(vcc, evaluationPlanId, interactionId, userId);
        else
            return Promise.resolve(true);
    })
        .then(function (data) {
        if (ok)
            return Promise.resolve(data);
        else
            return Promise.reject(data);
    })
        .catch(function (err) {
        return Promise.reject(err);
    });
}
